/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
 *
 * Copyright (C) 2007-2008 Lucian Langa <cooly@mips.edu.ms> 
 *  
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or 
 *  (at your option) any later version.
 *  
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <glib/gi18n-lib.h>
#include <string.h>
#include <stdio.h>
#include <fcntl.h>

#include <gtk/gtk.h>

#include <camel/camel-mime-message.h>
#include <camel/camel-folder.h>
#include <camel/camel-exception.h>
#include <camel/camel-multipart.h>
#include <camel/camel-stream-mem.h>

#include <mail/em-popup.h>
#include <e-util/e-mktemp.h>
#include <mail/em-config.h>

#include <libedataserver/e-account-list.h>
#include <misc/e-activity-handler.h>

#include "mail/em-utils.h"
#include "mail/mail-component.h"

#include "rspam.h"

struct rspam_stuff {
    EMConfigTargetPrefs *target;
	
    GtkWidget *option_menu;
    GtkWidget *option_menu2;
    GtkWidget *option_menu3;
    GtkWidget *option_menu4;
    GtkWidget *username;
    GtkWidget *password;
    GtkWidget *home;
    gchar *homepath;
    gchar *user;
    gchar *pass;
    gchar *spamcop_email;
};

void spamcop_report(CamelMimeMessage *msg, gchar *msgfile);
void taskbar_push_message(gchar *message);
void taskbar_pop_message(void);
void rs_online(CamelObject *o, void *event_data, void *data);

#define CNETS 			4
#define SPAMCOP_LIMIT 		50*1024
//temporary use spamassassin default address
//as spamcop guys did not respond
#define SPAMCOP_DEFAULT_EMAIL 	"spamassassin-submit@spam.spamcop.net"

#define RAZORLINE RAZOREP

//colaborative network	spam command line		ham command line	gconf flag name
//	     	         name
char *cnets[CNETS][4] = {
     {"razor",	"razor-report -f -d",  		"razor-revoke -f -d",	GCONF_KEY_ENABLE_RAZOR},
     {"dcc",  	"dccproc -H -t many -x 0",	NULL,			GCONF_KEY_ENABLE_DCC},
     {"spamcop",(gchar *)spamcop_report,	NULL,			GCONF_KEY_ENABLE_SPAMCOP},
     {"pyzor",  "pyzor -d report", 		"pyzor -d whitelist",	GCONF_KEY_ENABLE_PYZOR}};

gchar *pyzor_status = NULL;
GConfClient *rspam_gconf;
struct rspam_stuff *stuff;
gboolean rspam_online;

static void
readrazorconfig(gpointer data);

//spam functions wrapper
//for calling internal functions
void
intern_call(void (*p)(), CamelMimeMessage *msg, gchar *msgfile)
{
    p(msg, msgfile);
}

/* Save email content as a file */
gchar*
save_message(CamelMimeMessage *msg)
{
	CamelDataWrapper *containee;
	CamelStream *out;
	char *tmpdir = NULL, *name = NULL;
	tmpdir = e_mkdtemp("rspam-XXXXXX");
	if ( tmpdir == NULL)
	    return (gchar *)-1;

	name = g_build_filename(tmpdir, ".evo-attachment.rspam", NULL);
        out = camel_stream_fs_new_with_name(name, O_RDWR|O_CREAT, 0666);
        if (out == NULL)
                goto fail;

	// dump the whole message incl headers to report
        if (camel_data_wrapper_decode_to_stream((CamelDataWrapper *)msg, out) == -1
            || camel_stream_close(out) == -1) {
                camel_object_unref(out);
                goto fail;
        }
        camel_object_unref(out);
	return name;
	
fail:
	g_free(name);
	g_free(tmpdir);
	return (gchar *)-1;
}

void
update_stats(gboolean command)
{
	FILE *STF;
	gchar buf[512];
	int spam=0;
	gchar *stfile;
	
	//update stats
	if (command)
		stfile = g_build_filename(g_get_home_dir(), ".evolution", ".rspam", NULL);
	else
		stfile = g_build_filename(g_get_home_dir(), ".evolution", ".rham", NULL);
	STF = fopen(stfile, "r");
	if (!STF)
		goto create;
	fgets(buf, 512, STF);
	fclose(STF);
	spam = atoi(buf);
create:	spam++;
	STF = fopen(stfile, "w+");
	if (!STF)
		return;
	fprintf(STF, "%d", spam);
	fclose(STF);
}

gint
rspam_command(CamelMimeMessage *msg, gchar *name, gint command) //command: 1 - spam; 0 - ham
{
	gint i;
	gchar *cmd, *cmdp;
	int fds[2];
	pid_t pid;
	
	//later we have to check if we have a home and we're registered
	//remember to put -s when testing, in order no to clog cloudmark
	for (i=0; i < CNETS; i++)
	{
	    g_print("CNET: %s\n", cnets[i][0]);
	    if (gconf_client_get_bool (rspam_gconf, cnets[i][3], NULL))
	    {
		g_print("%s is enabled.\n", cnets[i][0]);
		if (command)	/*spam*/
		{
			if (!strncmp(cnets[i][0], "spamcop", 7))
			    intern_call((void *)cnets[i][1], msg, name);
			else
			{
				if (rspam_online)
				{
					if (!gconf_client_get_bool(rspam_gconf, GCONF_KEY_DRY_RUN, NULL))
					{
						cmdp = g_strconcat(cnets[i][1], " < %s", NULL);
		    				cmd = g_strdup_printf(cmdp, name);
    		    				system(cmd);
    		    				g_free(cmd);
    		    				g_free(cmdp);
						taskbar_push_message(_("Message reported as SPAM!"));
					}
					else
						taskbar_push_message(_("Message reported as SPAM! (dry mode!)"));
				}
				else
					taskbar_push_message(_("Offline mode! Not reporting SPAM!"));

			}
		}
		else		/*ham*/
		{
			if (rspam_online)
			{
				if (NULL != cnets[i][2])
				{
					if (!gconf_client_get_bool(rspam_gconf, GCONF_KEY_DRY_RUN, NULL))
					{
						cmdp = g_strconcat(cnets[i][2], " < %s", NULL);
						cmd = g_strdup_printf(cmdp, name);
    						system(cmd);
    						g_free(cmd);
    						g_free(cmdp);
					}
					taskbar_push_message(_("Message is no loger SPAM!"));
				}
				else
				{
					gchar *msg = 
					g_strdup_printf(_("Revoke operation not supported on %s!"),
										cnets[i][0]);
					taskbar_push_message(msg);
					g_free(msg);
#ifdef DEBUG
					g_print("revoke not supported!\n");
#endif
				}
			}
			else
				taskbar_push_message(_("Offline mode. Cannot revoke SPAM."));
				
		}
		update_stats(command);
	    }
	    else
		g_print("%s is disabled. Skip test.\n", cnets[i][0]);
	}
end:   		//let's just make sure we don't leave any junk on our way out,
    		//even if we're in trash pile
    		cmd = g_strdup_printf("rm -f %s", name);
    		system(cmd);
    		g_free(cmd);
		return 1;
}


void
spamcop_report(CamelMimeMessage *msg, gchar *msgfile)
{
    int tzone;
    time_t date, now;
    EAccount *account;
    gchar *from, *descr;
    CamelFolder *mail_folder;
    CamelMessageInfo *info;
    struct _send_data *send;
    GtkWidget *composer;
    CamelMimePart *part;
    CamelMimeMessage *new = camel_mime_message_new();
    gchar *tmsg;
    CamelMultipart *body = camel_multipart_new();
    CamelInternetAddress *addr;
    CamelStream *mem;
    gchar *sc_email;
    
    date = camel_mime_message_get_date_received(msg, &tzone);
    
    /*Convert to UTC*/
    date += (tzone / 100) * 60 * 60;
    date += (tzone % 100) * 60;
    
    time(&now);
    
    if (date && date < now - 2*86400)
    {
	taskbar_push_message(_("SpamCop: Message too old, not reporting mail older than 2 days."));
#ifdef DEBUG
	g_print("SpamCop: Message too old, not reporting mail older than 2 days.\n");
#endif
	g_print("now:%d\n", (guint)now - 2*86400);
	g_print("date:%d\n", (guint)date);
	goto fail;
    }
    
    //test this with multiple enabled/disabled accounts
    account = mail_config_get_default_account();
    if (!account->enabled)
    {
	e_error_run(NULL, "mail:send-no-account-enabled", NULL);
	goto fail;
    }
    
    // check and trim if message is above spamcop limit 50k
    // couldn't find better method at this time :-)
    mem = camel_stream_mem_new();
    camel_data_wrapper_decode_to_stream(CAMEL_DATA_WRAPPER(msg), CAMEL_STREAM(mem));
    if ( SPAMCOP_LIMIT < ((CamelStreamMem *) mem)->buffer->len)
    {
	tmsg = g_strndup( (const gchar *)((CamelStreamMem *) mem)->buffer->data, SPAMCOP_LIMIT);
	camel_object_unref(mem);
	camel_object_unref(msg);
	gchar *tmp = g_strconcat((gchar *)tmsg, "[message truncated by evolution RSPAM]", NULL);
	g_free(tmsg);
	mem = camel_stream_mem_new();
	camel_stream_write(mem, tmp, strlen(tmp));
	camel_stream_reset(mem);
	camel_object_unref(tmp);
	msg = camel_mime_message_new();
	camel_data_wrapper_construct_from_stream(CAMEL_DATA_WRAPPER(msg), mem);
	camel_object_unref(mem);
    }
    
    mail_folder = mail_component_get_folder(NULL, MAIL_COMPONENT_FOLDER_OUTBOX);
    camel_object_ref(mail_folder);
    
    info = camel_message_info_new(NULL);
    camel_message_info_set_flags(info, CAMEL_MESSAGE_SEEN, ~0);
    
    camel_mime_message_set_subject(new, "report spam");
    
    addr = camel_internet_address_new();
    sc_email = gconf_client_get_string(rspam_gconf, GCONF_KEY_SPAMCOP_EMAIL, NULL);
    camel_address_decode(CAMEL_ADDRESS(addr), 
	sc_email ? sc_email : SPAMCOP_DEFAULT_EMAIL);
    camel_mime_message_set_recipients(new, CAMEL_RECIPIENT_TYPE_TO, addr);
    camel_object_unref(addr);
    
    addr = camel_internet_address_new();
    camel_internet_address_add(addr, account->id->name, account->id->address);
    camel_mime_message_set_from(new, addr);
    camel_object_unref(addr);
    
    part = mail_tool_make_message_attachment(msg);
    camel_mime_part_set_content_type(part,
				    "message/rfc822;"
				    " x-spam-type=report");
    descr = g_strdup_printf("spam report via RSPAM-%s", VERSION);
    camel_mime_part_set_description(part, descr);
    camel_mime_part_set_disposition(part, "attachment");
    camel_medium_set_header(CAMEL_MEDIUM(part), "Content-Transfer-Encoding", "8bit");
//    camel_medium_set_header(part, "X-Spam-Relays-Trusted", "");
//    camel_medium_set_header(part, "X-Spam-Relays-Untrusted", "");
    
    camel_data_wrapper_set_mime_type(CAMEL_DATA_WRAPPER(body),
				    "multipart/mixed;"
				    "boundary=\"\"");
    camel_multipart_set_boundary(body, NULL);
    camel_multipart_add_part(body, part);
    camel_object_unref(part);
    
    /*more voodoo here*/
    
    if (!gconf_client_get_bool(rspam_gconf, GCONF_KEY_DRY_RUN, NULL))
    {
    	camel_medium_set_content_object(CAMEL_MEDIUM(new), CAMEL_DATA_WRAPPER(body));
    	camel_object_unref(body);
    
    	mail_append_mail(mail_folder, new, info, NULL, NULL);
	/*composer_send_queued_cb*/

    	update_stats(1);
    }
    
fail:	return;
    
}

guint
read_spam(void)
{
	gchar *stfile;
	FILE *STF;
	gchar buf[512];

	stfile = g_build_filename(g_get_home_dir(), ".evolution", ".rspam", NULL);
	STF = fopen(stfile, "r");
	if (!STF)
		return 0;
	fgets(buf, 512, STF);
	fclose(STF);
	return atoi(buf);
}

guint
read_ham(void)
{
	gchar *stfile;
	FILE *STF;
	gchar buf[512];

	stfile = g_build_filename(g_get_home_dir(), ".evolution", ".rham", NULL);
	STF = fopen(stfile, "r");
	if (!STF)
		return 0;
	fgets(buf, 512, STF);
	fclose(STF);
	return atoi(buf);
}

void org_gnome_sa_rspam(void *ep, EMPopupTargetSelect *t);

void
org_gnome_sa_rspam(void *ep, EMPopupTargetSelect *t)
{
	CamelMimeMessage *msg;
	CamelException ex = { 0 };
	gchar *name;
	gint i;
	
	while (gtk_events_pending())
                  gtk_main_iteration ();

	for (i = 0; i < (t->uids ? t->uids->len : 0); i++)
	{
		/* NB: use mail_get_messages, dont get the message in the main thread */
		msg = camel_folder_get_message(t->folder, g_ptr_array_index(t->uids, i), &ex);
		/* probably this isn't a good ideea */
		if (!msg)
			continue;
/*		if (msg == NULL) {
			camel_exception_clear(&ex);
			return;
		}*/
		
		name = save_message(msg);
		if (name)
		{
			rspam_command(msg, name, 1);
			g_free(name);
		}
		
		if (gconf_client_get_bool (rspam_gconf, GCONF_KEY_MOVE_JUNK, NULL))
		    camel_folder_set_message_flags(t->folder, g_ptr_array_index(t->uids, i),
			CAMEL_MESSAGE_JUNK|CAMEL_MESSAGE_SEEN,
			CAMEL_MESSAGE_JUNK|CAMEL_MESSAGE_SEEN);
		
		camel_object_unref(msg);
	}
	while (gtk_events_pending())
                  gtk_main_iteration ();
}

void org_gnome_sa_revoke(void *ep, EMPopupTargetSelect *t);

void
org_gnome_sa_revoke(void *ep, EMPopupTargetSelect *t)
{
	CamelMimeMessage *msg;
	CamelException ex = { 0 };
	gchar *name;
	gint i;

	while (gtk_events_pending())
                  gtk_main_iteration ();

	for (i = 0; i < (t->uids ? t->uids->len : 0); i++)
	{
	    msg = camel_folder_get_message(t->folder, g_ptr_array_index(t->uids, i), &ex);
	    if (!msg)
			continue;

    	    name = save_message(msg);
	    if (name)
	    {
		rspam_command(msg, name, 0);
		g_free(name);
	    }

	    if (gconf_client_get_bool (rspam_gconf, GCONF_KEY_MOVE_JUNK, NULL))
		camel_folder_set_message_flags(t->folder, g_ptr_array_index(t->uids, i),
		    CAMEL_MESSAGE_SEEN|CAMEL_MESSAGE_JUNK, CAMEL_MESSAGE_SEEN);

	    camel_object_unref(msg);
	}

	while (gtk_events_pending())
                  gtk_main_iteration ();
}


gchar* 
pyzor_discover_cb (GtkWidget *widget, gpointer data)
{
	GByteArray *output_buffer = g_byte_array_new ();
	int result, status, errnosav, fds[2], out_fds[2];
        CamelStream *stream;
        CamelStream *stream2;
        char *program;
        pid_t pid;
        int i=0;
	char *argv[30];

	if (!rspam_online)
		return 0;

	argv[i++] = "pyzor";
        argv[i++] = "ping";
        argv[i] = NULL;

        if (output_buffer && pipe (out_fds) == -1) {
                return 0;
        }

	if (!(pid = fork ())) {
                /* child process */
                int maxfd, fd;

                if (dup2 (out_fds[1], STDOUT_FILENO) == -1)
                        _exit (1 & 0377);
                if (output_buffer)
                        close (out_fds [1]);

                setsid ();

                maxfd = sysconf (_SC_OPEN_MAX);
                for (fd = 3; fd < maxfd; fd++)
                        fcntl (fd, F_SETFD, FD_CLOEXEC);

                execvp (argv[0], argv);
                _exit (1 & 0377);
        } else if (pid < 0) {
                return 0;
        }

        if (output_buffer)
                close (out_fds [1]);

	 if (output_buffer) {
                CamelStreamMem *memstream;

                stream = camel_stream_fs_new_with_fd (out_fds[0]);

                memstream = (CamelStreamMem *) camel_stream_mem_new ();
                camel_stream_mem_set_byte_array (memstream, output_buffer);

                camel_stream_write_to_stream (stream, (CamelStream *) memstream);
                camel_object_unref (stream);
                g_byte_array_append (output_buffer, "", 1);

                printf ("child process output: %s len: %d\n", output_buffer->data, output_buffer->len);
        }
	gchar *tmp = g_strdup(output_buffer->data);

	return tmp;
}

static void
discover_cb (GtkWidget *widget, gpointer data)
{
	struct rspam_stuff *stuff = (struct rspam_stuff *) data;
	gchar *cmd;
	if (!rspam_online)
		return;

	const gchar *home = stuff->homepath;
	const gchar *user = stuff->user;
	const gchar *pass = stuff->pass;
	if (home && user && pass)
		cmd = g_strdup_printf("razor-admin -home %s -discover -user %s -pass %s", 
		home, user, pass);
	else
		cmd = g_strdup("razor-admin -discover"); 
#ifdef DEBUG
	g_print("Razor discover - cmd:%s\n", cmd);
#endif
	system(cmd);
	g_free(cmd);
}

static void
register_cb (GtkWidget *widget, gpointer data)
{
	struct rspam_stuff *stuff = (struct rspam_stuff *) data;
	gchar *cmd;
	if (!rspam_online)
		return;

	const gchar *home = gtk_entry_get_text(GTK_ENTRY(stuff->home));
	const gchar *user = gtk_entry_get_text(GTK_ENTRY(stuff->username));
	const gchar *pass = gtk_entry_get_text(GTK_ENTRY(stuff->password));
	cmd = g_strdup_printf("razor-admin -d -home %s -register -user %s -pass %s", 
		home, user, pass);
#ifdef DEBUG
	g_print("Razor register - cmd:%s\n", cmd);
#endif
	system(cmd);
	g_free(cmd);
	readrazorconfig(stuff);
}

static void
razor_home_cb (GtkWidget *widget, gpointer data)
{
    struct rspam_stuff *stuff = (struct rspam_stuff *) data;
    const gchar *razor_home = gtk_entry_get_text(GTK_ENTRY(widget));
		
    /* Save the new setting to gconf */
    gconf_client_set_string (stuff->target->gconf, GCONF_KEY_RAZOR_HOME, razor_home, NULL);
}

static void
spamcop_email_cb (GtkWidget *widget, gpointer data)
{
    struct rspam_stuff *stuff = (struct rspam_stuff *) data;
    const gchar *spamcop_email = gtk_entry_get_text(GTK_ENTRY(widget));
		
    /* Save the new setting to gconf */
    gconf_client_set_string (stuff->target->gconf, GCONF_KEY_SPAMCOP_EMAIL, spamcop_email, NULL);
}

static void
enable_dcc_cb (GtkWidget *widget, gpointer data)
{
    struct rspam_stuff *stuff = (struct rspam_stuff *) data;
    gboolean active = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget));
		
    /* Save the new setting to gconf */
    gconf_client_set_bool (stuff->target->gconf, GCONF_KEY_ENABLE_DCC, active, NULL);
}

static void
enable_razor_cb (GtkWidget *widget, gpointer data)
{
    struct rspam_stuff *stuff = (struct rspam_stuff *) data;
    gboolean active = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget));
		
    /* Save the new setting to gconf */
    gconf_client_set_bool (stuff->target->gconf, GCONF_KEY_ENABLE_RAZOR, active, NULL);
}

static void
enable_spamcop_cb (GtkWidget *widget, gpointer data)
{
    struct rspam_stuff *stuff = (struct rspam_stuff *) data;
    gboolean active = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget));
		
    /* Save the new setting to gconf */
    gconf_client_set_bool (stuff->target->gconf, GCONF_KEY_ENABLE_SPAMCOP, active, NULL);
}

static void
enable_movejunk_cb (GtkWidget *widget, gpointer data)
{
    struct rspam_stuff *stuff = (struct rspam_stuff *) data;
    gboolean active = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget));
		
    /* Save the new setting to gconf */
    gconf_client_set_bool (stuff->target->gconf, GCONF_KEY_MOVE_JUNK, active, NULL);
}

static void
enable_dryrun_cb (GtkWidget *widget, gpointer data)
{
    struct rspam_stuff *stuff = (struct rspam_stuff *) data;
    gboolean active = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget));
		
    /* Save the new setting to gconf */
    gconf_client_set_bool (stuff->target->gconf, GCONF_KEY_DRY_RUN, active, NULL);
}

static void
enable_pyzor_cb (GtkWidget *widget, gpointer data)
{
    struct rspam_stuff *stuff = (struct rspam_stuff *) data;
    gboolean active = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget));
		
    /* Save the new setting to gconf */
    gconf_client_set_bool (stuff->target->gconf, GCONF_KEY_ENABLE_PYZOR, active, NULL);
}

static void
readrazorconfig(gpointer data)
{
	gchar buf[512];
	char **str2;
	FILE *file;
	gchar *key, *path;
	gchar *str, *cmd;
	EAccount *account;

	struct rspam_stuff *stuff = (struct rspam_stuff *) data;

	//we might had razor before, so try to fetch it's home
	//otherwise we'll build ovious razor home from homedir
	key = gconf_client_get_string(rspam_gconf, GCONF_KEY_RAZOR_HOME, NULL);
	if (!key)
	{
		str = g_build_filename(g_get_home_dir(), ".razor", "identity", NULL);
		gchar *tmp = g_path_get_dirname(str);
    		gconf_client_set_string (rspam_gconf, GCONF_KEY_RAZOR_HOME, tmp, NULL);
		g_free(tmp);
		key = gconf_client_get_string(rspam_gconf, GCONF_KEY_RAZOR_HOME, NULL);
	}
	else
	{
		//we  have a razor home stored
		str = g_build_filename(key, "identity", NULL);
	}
	stuff->homepath = key;

	//try to see if files really exists	
	file = fopen(str, "rb");
	if (!file)
	{
		account = mail_config_get_default_account();
		g_print("account:%s\n", account->id->address);
		//identity file does not exist, let's try home
		path = g_path_get_dirname(str);
		if (!g_file_test(path, G_FILE_TEST_IS_DIR))
		{
			//create
			cmd = g_strdup_printf("razor-admin -d -home %s -create",
					path);
		g_print("%s\n", cmd);
    		system(cmd);
		g_free(cmd);
		}
		//register
		cmd = g_strdup_printf("razor-admin -d -home %s -register", 
				path);
		g_print("%s\n", cmd);
    		system(cmd);
		g_free(cmd);
		//file must now be created
		//to we should check status
		file = fopen(str, "rb");
		if (!file)
		{
			stuff->user = NULL;
			stuff->pass = NULL;
			return;
		}
	}

	while(fgets(buf, 512, file))
	{
		g_strstrip(buf);
		str2 = g_strsplit(buf, "=", 0);
		g_strstrip(str2[1]);
		if (!strncmp(str2[0], "user", 4))
			stuff->user = str2[1];
		if (!strncmp(str2[0], "pass", 4))
			stuff->pass = str2[1];
	}
	fclose(file);
}

void
taskbar_push_message(gchar *message)
{
        EActivityHandler *activity_handler = 
			mail_component_peek_activity_handler (mail_component_peek ());
        e_activity_handler_set_message(activity_handler, message);
}

void
taskbar_pop_message(void)
{
        EActivityHandler *activity_handler = 
			mail_component_peek_activity_handler (mail_component_peek ());
        e_activity_handler_unset_message(activity_handler);
}

//=================
//== guification ==
//=================

GtkWidget *
rspam_page_factory (EPlugin *ep, EConfigHookItemFactoryData *hook_data)
{
    struct rspam_stuff *stuff;
    EMConfigTargetPrefs *target = (EMConfigTargetPrefs *) hook_data->config->target;
    GtkWidget *page;
    GtkWidget *tab_label;
    GtkWidget *frame;
    GtkWidget *frame_label;
    GtkWidget *padding_label;
    GtkWidget *hbox;
    GtkWidget *inner_vbox;
    GtkWidget *check;
    GtkWidget *option;
    GtkWidget *check_gaim;
    GtkWidget *button;
    GtkWidget *button2;
    GtkWidget *entry;
    GtkWidget *entry1,*entry2,*entry3;
    GtkWidget *label1, *label2, *label3;
    GtkWidget *table1;
    gchar     *str;
    GtkTooltips *tips;
   
    tips = gtk_tooltips_new ();
						    
    /* A structure to pass some stuff around */
    stuff = g_new0 (struct rspam_stuff, 100);
    stuff->target = target;

    readrazorconfig(stuff);
							
    /* Create a new notebook page */
    page = gtk_vbox_new (FALSE, 0);
    GTK_CONTAINER (page)->border_width = 12;
    tab_label = gtk_label_new (_("Report Spam"));
    gtk_notebook_append_page (GTK_NOTEBOOK (hook_data->parent), page, tab_label);
	    
    /* Frame */
    frame = gtk_vbox_new (FALSE, 6);
    gtk_box_pack_start (GTK_BOX (page), frame, FALSE, FALSE, 0);
					
	/* "Filtering Network" */
	frame_label = gtk_label_new ("");
        str = g_strdup_printf ("<span weight=\"bold\">%s</span>", _("Filtering Network"));
    	gtk_label_set_markup (GTK_LABEL (frame_label), str);
    	g_free (str);
	GTK_MISC (frame_label)->xalign = 0.0;
        gtk_box_pack_start (GTK_BOX (frame), frame_label, FALSE, FALSE, 0);

        /* Indent/padding */
        hbox = gtk_hbox_new (FALSE, 12);
	gtk_box_pack_start (GTK_BOX (frame), hbox, FALSE, TRUE, 0);
        padding_label = gtk_label_new ("");
	gtk_box_pack_start (GTK_BOX (hbox), padding_label, FALSE, FALSE, 0);
        inner_vbox = gtk_vbox_new (FALSE, 6);
	gtk_box_pack_start (GTK_BOX (hbox), inner_vbox, FALSE, FALSE, 0);

	check = gtk_check_button_new_with_mnemonic (_("_Vipul's Razor (Razor)"));
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (check), gconf_client_get_bool (target->gconf, GCONF_KEY_ENABLE_RAZOR, NULL));
	g_signal_connect (GTK_TOGGLE_BUTTON (check), "toggled", G_CALLBACK (enable_razor_cb), stuff);
	gtk_tooltips_set_tip (GTK_TOOLTIPS (tips), check,
		_("Vipul's Razor is a distributed, collaborative, spam detection and filtering network. Through user contribution, Razor establishes a distributed and constantly updating catalogue of spam in propagation."), NULL);
	gtk_box_pack_start (GTK_BOX (inner_vbox), check, FALSE, FALSE, 0);
	check = gtk_check_button_new_with_mnemonic (_("_Pyzor"));
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (check), gconf_client_get_bool (target->gconf, GCONF_KEY_ENABLE_PYZOR, NULL));
	g_signal_connect (GTK_TOGGLE_BUTTON (check), "toggled", G_CALLBACK (enable_pyzor_cb), stuff);
	gtk_box_pack_start (GTK_BOX (inner_vbox), check, FALSE, FALSE, 0);
	gtk_tooltips_set_tip (GTK_TOOLTIPS (tips), check,
		_("Python implementation of Razor network."), NULL);
        check = gtk_check_button_new_with_mnemonic (_("_Distributed Checksum Clearinghouse (DCC)"));
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (check), gconf_client_get_bool (target->gconf, GCONF_KEY_ENABLE_DCC, NULL));
	g_signal_connect (GTK_TOGGLE_BUTTON (check), "toggled", G_CALLBACK (enable_dcc_cb), stuff);
	gtk_tooltips_set_tip (GTK_TOOLTIPS (tips), check,
		_("A DCC server totals reports of checksums of messages from clients and answers queries about the total counts for checksums of mail messages."), NULL);
	gtk_box_pack_start (GTK_BOX (inner_vbox), check, FALSE, FALSE, 0);
	check = gtk_check_button_new_with_mnemonic (_("_SpamCop.Net"));
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (check), gconf_client_get_bool (target->gconf, GCONF_KEY_ENABLE_SPAMCOP, NULL));
	g_signal_connect (GTK_TOGGLE_BUTTON (check), "toggled", G_CALLBACK (enable_spamcop_cb), stuff);
	gtk_box_pack_start (GTK_BOX (inner_vbox), check, FALSE, FALSE, 0);
	

	check = gtk_check_button_new_with_mnemonic (_("Move to Junk folder after report"));
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (check), gconf_client_get_bool (target->gconf, GCONF_KEY_MOVE_JUNK, NULL));
	g_signal_connect (GTK_TOGGLE_BUTTON (check), "toggled", G_CALLBACK (enable_movejunk_cb), stuff);

	gtk_tooltips_set_tip (GTK_TOOLTIPS (tips), check,
		_("Moves message to Junk Folder after spam report without affecting spam learing (see SA plugin)"), NULL);
	gtk_box_pack_start (GTK_BOX (frame), check, FALSE, FALSE, 0);
	
	check = gtk_check_button_new_with_mnemonic (_("Dry-run"));
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (check), gconf_client_get_bool (target->gconf, GCONF_KEY_DRY_RUN, NULL));
	g_signal_connect (GTK_TOGGLE_BUTTON (check), "toggled", G_CALLBACK (enable_dryrun_cb), stuff);
	gtk_tooltips_set_tip (GTK_TOOLTIPS (tips), check,
				 _("Go to all phases of spam reporting but do not actualy connect to reporting servers."), NULL);
	gtk_box_pack_start (GTK_BOX (frame), check, FALSE, FALSE, 0);
	
	/* "Razor Settings" */
	frame_label = gtk_label_new ("");
        str = g_strdup_printf ("<span weight=\"bold\">%s</span>", _("Razor Settings"));
    	gtk_label_set_markup (GTK_LABEL (frame_label), str);
    	g_free (str);
	GTK_MISC (frame_label)->xalign = 0.0;
        gtk_box_pack_start (GTK_BOX (frame), frame_label, FALSE, FALSE, 0);

	table1 = gtk_table_new (4, 2, FALSE);
	gtk_container_add (GTK_CONTAINER (frame), table1);
	gtk_table_set_row_spacings (GTK_TABLE (table1), 4);

  	label1 = gtk_label_new_with_mnemonic (_("Razor _Home: "));
	gtk_table_attach (GTK_TABLE (table1), label1, 0, 1, 0, 1,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
	gtk_misc_set_alignment (GTK_MISC (label1), 0, 0.5);
  
	label2 = gtk_label_new (_("Username: "));
	gtk_table_attach (GTK_TABLE (table1), label2, 0, 1, 1, 2,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
	gtk_misc_set_alignment (GTK_MISC (label2), 0, 0.5);
  
	label3 = gtk_label_new (_("Password: "));
	gtk_table_attach (GTK_TABLE (table1), label3, 0, 1, 2, 3,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
	gtk_misc_set_alignment (GTK_MISC (label3), 0, 0.5);
  
	entry1 = gtk_entry_new ();
	gtk_table_attach (GTK_TABLE (table1), entry1, 1, 2, 0, 1,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
	gtk_entry_set_invisible_char (GTK_ENTRY (entry1), 8226);
	gtk_entry_set_text(GTK_ENTRY(entry1), gconf_client_get_string(target->gconf, GCONF_KEY_RAZOR_HOME, NULL));
	g_signal_connect (GTK_ENTRY (entry1), "changed", G_CALLBACK (razor_home_cb), stuff);
	stuff->home = entry1;
  
	entry2 = gtk_entry_new ();
	gtk_table_attach (GTK_TABLE (table1), entry2, 1, 2, 1, 2,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
	gtk_entry_set_text(GTK_ENTRY(entry2), (gpointer)stuff->user);
	gtk_entry_set_invisible_char (GTK_ENTRY (entry2), 8226);
	stuff->username = entry2;
  
	entry3 = gtk_entry_new ();
	gtk_table_attach (GTK_TABLE (table1), entry3, 1, 2, 2, 3,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
	gtk_entry_set_visibility (GTK_ENTRY (entry3), FALSE);
	gtk_entry_set_text(GTK_ENTRY(entry3), (gpointer)stuff->pass);
	gtk_entry_set_invisible_char (GTK_ENTRY (entry3), 8226);
	gtk_table_attach (GTK_TABLE (table1), entry3, 1, 2, 2, 3,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
	stuff->password = entry3;
  
	button = gtk_button_new_with_mnemonic(_("_Register"));
	gtk_table_attach (GTK_TABLE (table1), button, 0, 1, 3, 4,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
        gtk_misc_set_alignment (GTK_MISC (button), 0, 0.5);
	g_signal_connect (GTK_BUTTON (button), "clicked", G_CALLBACK (register_cb), stuff);
	gtk_tooltips_set_tip (GTK_TOOLTIPS (tips), button,
		_("Registers a new identity, used for authenticating with Razor Nomination Servers. razor-admin is required in $PATH."), NULL);
        gtk_box_pack_start (GTK_BOX (frame), table1, FALSE, FALSE, 0);
	
	button2 = gtk_button_new_with_mnemonic(_("_Discover"));
	gtk_table_attach (GTK_TABLE (table1), button2, 0, 1, 4, 5,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
        gtk_misc_set_alignment (GTK_MISC (button2), 0, 0.5);
	g_signal_connect (GTK_BUTTON (button2), "clicked", G_CALLBACK (discover_cb), stuff);
	gtk_tooltips_set_tip (GTK_TOOLTIPS (tips), button2,
		_("Force discovery. This will create server.*.lst files in Razor Home. razor-admin is required in $PATH"), NULL);
        gtk_box_pack_start (GTK_BOX (frame), table1, FALSE, FALSE, 0);

	
	/* "SpamCop Settings" */
	frame_label = gtk_label_new ("");
        str = g_strdup_printf ("<span weight=\"bold\">%s</span>", _("SpamCop Settings"));
    	gtk_label_set_markup (GTK_LABEL (frame_label), str);
    	g_free (str);
	GTK_MISC (frame_label)->xalign = 0.0;
        gtk_box_pack_start (GTK_BOX (frame), frame_label, FALSE, FALSE, 0);

	table1 = gtk_table_new (4, 2, FALSE);
	gtk_container_add (GTK_CONTAINER (frame), table1);
	gtk_table_set_row_spacings (GTK_TABLE (table1), 4);

  	label1 = gtk_label_new_with_mnemonic (_("Report Address: "));
	gtk_table_attach (GTK_TABLE (table1), label1, 0, 1, 0, 1,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
	gtk_misc_set_alignment (GTK_MISC (label1), 0, 0.5);
	
	entry1 = gtk_entry_new ();
	gtk_table_attach (GTK_TABLE (table1), entry1, 1, 2, 0, 1,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
	gtk_entry_set_invisible_char (GTK_ENTRY (entry1), 8226);
	gchar *sc_email = gconf_client_get_string(target->gconf, GCONF_KEY_SPAMCOP_EMAIL, NULL);
	gtk_entry_set_text(GTK_ENTRY(entry1),
	    (sc_email ? sc_email : SPAMCOP_DEFAULT_EMAIL));
	g_signal_connect (GTK_ENTRY (entry1), "changed", G_CALLBACK (spamcop_email_cb), stuff);
	stuff->spamcop_email = sc_email ? sc_email : "rspam-submit@spam.spamcop.net";
	gtk_tooltips_set_tip (GTK_TOOLTIPS (tips), entry1,
		_("By default plugin uses a default email address. However you are encouraged to register to http://spamcop.net because your report will weight more using your account."), NULL);
	
	gtk_box_pack_start (GTK_BOX (frame), table1, FALSE, FALSE, 0);
	
/*	frame_label = gtk_label_new ("");
        str = g_strdup_printf ("<span weight=\"bold\">%s</span>", _("Pyzor Statistics"));
        gtk_label_set_markup (GTK_LABEL (frame_label), str);
        g_free (str);
        GTK_MISC (frame_label)->xalign = 0.0;
        gtk_box_pack_start (GTK_BOX (frame), frame_label, FALSE, FALSE, 0);

        table1 = gtk_table_new (1, 2, FALSE);
        gtk_container_add (GTK_CONTAINER (frame), table1);
        gtk_table_set_row_spacings (GTK_TABLE (table1), 4);

        label1 = gtk_label_new_with_mnemonic (_("Server: "));
        gtk_table_attach (GTK_TABLE (table1), label1, 0, 1, 0, 1,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
        gtk_misc_set_alignment (GTK_MISC (label1), 0, 0.5);

        label1 = gtk_label_new_with_mnemonic (pyzor_status);
        gtk_table_attach (GTK_TABLE (table1), label1, 1, 2, 0, 1,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
        gtk_misc_set_alignment (GTK_MISC (label1), 0, 0.5);

        gtk_box_pack_start (GTK_BOX (frame), table1, FALSE, FALSE, 0);*/

	frame_label = gtk_label_new ("");
        str = g_strdup_printf ("<span weight=\"bold\">%s</span>", _("Spam Statistics"));
        gtk_label_set_markup (GTK_LABEL (frame_label), str);
        g_free (str);
        GTK_MISC (frame_label)->xalign = 0.0;
        gtk_box_pack_start (GTK_BOX (frame), frame_label, FALSE, FALSE, 0);

        table1 = gtk_table_new (2, 2, FALSE);
        gtk_container_add (GTK_CONTAINER (frame), table1);
        gtk_table_set_row_spacings (GTK_TABLE (table1), 4);

        label1 = gtk_label_new_with_mnemonic (_("Spam reported: "));
        gtk_table_attach (GTK_TABLE (table1), label1, 0, 1, 0, 1,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
        gtk_misc_set_alignment (GTK_MISC (label1), 0, 0.5);

        label1 = gtk_label_new_with_mnemonic (_("Spam revoked: "));
        gtk_table_attach (GTK_TABLE (table1), label1, 0, 1, 1, 2,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
        gtk_misc_set_alignment (GTK_MISC (label1), 0, 0.5);

        guint spam = read_spam();
        gchar *tmp = g_strdup_printf(_("%d messages"), spam?spam:0);
        label1 = gtk_label_new_with_mnemonic (tmp);
        gtk_table_attach (GTK_TABLE (table1), label1, 1, 2, 0, 1,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
        gtk_misc_set_alignment (GTK_MISC (label1), 0, 0.5);
        g_free(tmp);

        spam = read_ham();
        tmp = g_strdup_printf(_("%d messages"), spam?spam:0);
        label1 = gtk_label_new_with_mnemonic (tmp);
        gtk_table_attach (GTK_TABLE (table1), label1, 1, 2, 1, 2,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
        gtk_misc_set_alignment (GTK_MISC (label1), 0, 0.5);
        g_free(tmp);

        gtk_box_pack_start (GTK_BOX (frame), table1, FALSE, FALSE, 0);
	
	/* Clean up */
//	g_signal_connect (page, "destroy", G_CALLBACK (cleanup_cb), stuff);

	gtk_widget_show_all (page);
																																							    
        return page;
}								                            			                            

void org_gnome_sa_rspam_startup(void *ep, EMPopupTargetSelect *t);

void org_gnome_sa_rspam_startup(void *ep, EMPopupTargetSelect *t)
{
}

void
rs_online(CamelObject *o, void *event_data, void *data)
{
        rspam_online =  camel_session_is_online (o);
}

int e_plugin_lib_enable(EPluginLib *ep, int enable);

int
e_plugin_lib_enable(EPluginLib *ep, int enable)
{
	if (enable) {
		bindtextdomain(GETTEXT_PACKAGE, LOCALEDIR);
		bind_textdomain_codeset(GETTEXT_PACKAGE, "UTF-8");
		g_print("Rspam Plugin enabled\n");

		camel_object_hook_event((void *)mail_component_peek_session(NULL),
                                 "online", rs_online, NULL);

    		rspam_gconf = gconf_client_get_default();
    		/* A structure to pass some stuff around */
    		stuff = g_new0 (struct rspam_stuff, 100);

    		readrazorconfig(stuff);
		taskbar_push_message(_("Performing discovery!"));
		/*discover pyzor*/
		if (gconf_client_get_bool (rspam_gconf, GCONF_KEY_ENABLE_PYZOR, NULL))
		pyzor_status = pyzor_discover_cb(NULL, NULL);
		/*discover razor*/
		if (gconf_client_get_bool (rspam_gconf, GCONF_KEY_ENABLE_RAZOR, NULL))
			discover_cb(NULL, stuff);
		taskbar_pop_message();
	} else {
		printf("Rspam Plugin disabled\n");
	}

	return 0;
}
